import timeit
import logging
import requests
from requests.auth import HTTPBasicAuth
from requests.exceptions import ConnectionError, Timeout
import json
import os
from nuaal.definitions import DATA_PATH
from nuaal.utils import get_logger


class RestBase2(object):
    """
    Parent class for all REST-like connections
    """
    def __init__(self, url, username=None, password=None, api_base_path=None, verify_ssl=False, DEBUG=False, con_type=None):
        """

        :param url: URL or IP address of the target machine
        :param username: Username for authentication
        :param password: Password for authentication
        :param api_base_path: Base path for the API resource, such as "/api/v1"
        :param verify_ssl: Enable SSL certificate verification. For self-signed certificates, set this to False
        :param DEBUG: Enable debugging output
        :param con_type: String representation of connection type, set by child classes
        """
        # Disable logging for external libraries
        logging.getLogger("requests").setLevel(logging.WARNING)
        logging.getLogger("urllib3").setLevel(logging.WARNING)

        self.url = url
        self.logger = get_logger(name="{}-{}".format(con_type if con_type else 'REST', self.url), DEBUG=DEBUG)
        self.username = username
        self.password = password
        self.api_base_path = api_base_path
        self.verify_ssl = verify_ssl
        self.DEBUG = DEBUG
        self.path_base = self.url + self.api_base_path
        self.headers = {"Content-type": "application/json"}
        self.common_headers = {"headers": self.headers, "verify": self.verify_ssl}
        self.authorized = False



    def _initialize(self):
        """
        Function for credentials loading and session preparation. Might be overwritten in child classes

        :return: ``None``
        """
        if not self.verify_ssl:
            # Disable certificate warning
            try:
                requests.packages.urllib3.disable_warnings()
            except:
                self.logger.warning(msg="Failed to disable Certificate Warnings")
        if self.username is None or self.password is None:
            self.logger.info(msg="No credentials provided, using cached instead.")
            if self._load_credentials():
                self.logger.info(msg="Credentials successfully loaded.")
            else:
                self.logger.info(msg="Failed to load valid credentials.")
        else:
            self._authorize()

    def _authorize(self):
        """
        Connection-specific function for handling authorization. Overridden by child classes

        :return: ``None``
        """
        pass

    def _store_data(self, data, path):
        path = os.path.join(DATA_PATH, path)
        try:
            with open(file=path, mode="w") as f:
                json.dump(obj=data, fp=f, indent=2)
        except Exception as e:
            self.logger.error(msg="Could not store data to file '{}'. Exception: {}".format(path, repr(e)))

    def _load_data(self, path):
        data = None
        path = os.path.join(DATA_PATH, path)
        try:
            with open(file=path, mode="r") as f:
                data = json.load(fp=f)
        except Exception as e:
            self.logger.error(msg="Could not load data from file '{}'. Exception: {}".format(path, repr(e)))
        finally:
            return data

    def _store_credentials(self):
        """
        Connection-specific function for handling authorization. Overridden by child classes

        :return: ``None``
        """
        pass

    def _load_credentials(self):
        """
        Connection-specific function for handling authorization. Overridden by child classes

        :return: ``None``
        """
        pass

    def request(self, method="GET", path="", raw=False, **kwargs):
        if not self.authorized:
            try:
                self._authorize()
            except Exception as e:
                self.logger.critical("Failed to authorize.")
            finally:
                if not self.authorized:
                    return None, None
        response = None
        url = self.path_base + path if path[0] == "/" else path
        try:
            response = requests.request(method=method, url=url, **kwargs, **self.common_headers)
        except requests.exceptions.ConnectionError:
            self.logger.critical(msg="Could not connect to {}".format(self.url))
        if raw:
            return response, response.status_code
        else:
            status_code = response.status_code
            if status_code == 200:
                return response.json(), status_code



import re
import json
from nuaal.definitions import DATA_PATH
from nuaal.utils import get_logger
from nuaal.Parsers.PatternsLib_Old import Patterns

class RegexBuilder:
    def __init__(self, device_type, DEBUG=False):
        self.device_type = device_type
        self.logger = get_logger(name="RegexBuilder", DEBUG=DEBUG)

    def _flags_map(self, flag_str):
        if not isinstance(flag_str, str):
            self.logger.error(msg="Given parameter 'flag_str' is not a string.")
        flags_dict = {
            "multiline": 40,
            "dotall": re.DOTALL
        }
        try:
            return flags_dict[flag_str]
        except KeyError:
            self.logger.error(msg="Flag '{}' is not a recognized regex flag.".format(flag_str))
            return None

    def _compile_pattern(self, pattern_dict):
        if not isinstance(pattern_dict, dict):
            self.logger.error(msg="Given parameter 'pattern_dict' is not a dictionary.")
        # Take pattern and flags from dictionary and return compiled pattern
        try:
            return re.compile(pattern=pattern_dict["pattern"], flags=pattern_dict["flags"])
        except Exception as e:
            # TODO: More specific exceptions
            self.logger.error(msg="Encountered exception when compiling pattern '{}'. Exception: {}".format(pattern_dict["pattern"], repr(e)))

    def _decompile_pattern(self, pattern):
        if not isinstance(pattern, re.Pattern):
            self.logger.error(msg="Given pattern is not a compiled re pattern.")
        pattern_string = pattern.pattern
        pattern_dict = {"pattern": pattern_string, "flags": 0}
        if "MULTILINE" in pattern.__str__():
            pattern_dict["flags"] = 40
        elif "DOTALL" in pattern.__str__():
            pattern_dict["flags"] = 48
        return pattern_dict

    def _store_json(self, pattern_data):
        command = pattern_data["command"]
        command_underscored = command.replace(" ", "_")
        with open(file="{}/patterns/{}/{}.json".format(DATA_PATH, self.device_type, command_underscored), mode="w") as file:
            try:
                json.dump(obj=pattern_data, fp=file, indent=2)
                self.logger.info(msg="Successfully stored JSON pattern in {}/{}.json".format(self.device_type, command_underscored))
            except Exception as e:
                self.logger.error(msg="Encountered exception when trying to save JSON pattern. Exception: {}".format(repr(e)))


if __name__ == '__main__':
    device_type = "cisco_ios"
    rb = RegexBuilder(device_type=device_type)

    pattern_dict = {
        "command": "show lldp neighbors detail",
        "pattern": r"^Enabled Capabilities: (?P<enCapabilities>\S+)$",
        "flags": re.MULTILINE
    }
    pattern = rb._compile_pattern(pattern_dict=pattern_dict)
    decompiled_pattern = rb._decompile_pattern(pattern=pattern)
    print(json.dumps(obj=decompiled_pattern, indent=2))
    pattern_dict.update(decompiled_pattern)
    print(pattern_dict)





from nuaal.utils import get_logger
from nuaal.definitions import DATA_PATH
import json


class Topology(object):
    def __init__(self, DEBUG=False):
        self.DEBUG = DEBUG
        self.logger = get_logger(name="Topology", DEBUG=DEBUG)
        self.topology = {"nodes": [], "links": []}

    def next_ui(self):
        id_couter = 0
        next_topo = {"nodes": [], "links": []}
        id_map = {}
        for node in self.topology["nodes"]:
            id_map[node] = id_couter
            next_topo["nodes"].append({"id": id_couter, "name": node})
            id_couter += 1
        for link in self.topology["links"]:
            next_topo["links"].append({"source": id_map[link["sourceNode"]], "target": id_map[link["targetNode"]]})
        return next_topo


class CliTopology(Topology):
    def __init__(self, DEBUG=False):
        super(CliTopology, self).__init__(DEBUG=DEBUG)

    def build_topology(self, data):
        if isinstance(data, list):
            new_data = {}
            for device in data:
                try:
                    new_data[device["hostname"]] = device["neighbors"]
                except KeyError:
                    self.logger.error(msg="Device data do not contain required keys.")
                    continue
            data = new_data
        
        self.logger.info(msg="Building topology based on {} visited devices.".format(len(data)))
        all_nodes = []
        all_links = []
        for device_id, neighbors in data.items():
            links = self._get_links(device_id=device_id, neighbors=neighbors)
            if device_id not in all_nodes:
                all_nodes.append(device_id)
            for neighbor_id in [x["targetNode"] for x in links]:
                if neighbor_id not in all_nodes:
                    all_nodes.append(neighbor_id)
            for link in links:
                if link not in all_links and self._reverse_link(link) not in all_links:
                    all_links.append(link)
        self.topology["links"] = all_links
        self.topology["nodes"] = all_nodes
        self.logger.info(msg="Discovered total of {} nodes and {} links".format(len(all_nodes), len(all_links)))

    def _get_links(self, device_id, neighbors):
        links = []
        for neighbor in neighbors:
            link = {
                "sourceNode": device_id,
                "sourceInterface": neighbor["localInterface"],
                "targetNode": neighbor["hostname"],
                "targetInterface": neighbor["remoteInterface"]
            }
            links.append(link)
        return links

    def _reverse_link(self, link):
        reverse_link = {
            "sourceNode": link["targetNode"],
            "sourceInterface": link["targetInterface"],
            "targetNode": link["sourceNode"],
            "targetInterface": link["sourceInterface"]
            }
        return reverse_link


from nuaal.utils import get_logger
import copy


class Filter:
    """
    Object for making filtering of required field easier.
    """
    def __init__(self, required={}, excluded={}, exact_match=True, DEBUG=False):
        """
        This class finds elements in JSON-like format based on provided values.

        :param dict required: Dictionary of required keys. Example: `{"key1": "value1"}` will return only those dictionaries, which key `"key1"` \
               contains the value `"value1"`. You can also use list of required values, such as `{"key1": ["value1", "value2"]}` together with option \
               ``exact_match=False``.

        :param dict excluded: Dictionary of excluded keys. Example: `{"key1": "value1"}` will return only those dictionaries, which key `"key1"` \
               does not contain the value `"value1"`. You can also use list of excluded values, such as `{"key1": ["value1", "value2"]}` together with option \
               ``exact_match=False``.

        :param bool exact_match: Specifies whether the filter value must match exactly, or partially.
        :param DEBUG: Enables/disables debugging output.
        """
        self.logger = get_logger(name="Filter", DEBUG=DEBUG)
        self.required = None
        self.excluded = None
        if isinstance(required, dict):
            self.required = required
        else:
            self.required = {}
            self.logger.error(msg="Required is not a dictionary!")
        if isinstance(excluded, dict):
            self.excluded = excluded
        else:
            self.excluded = {}
            self.logger.error(msg="Excluded is not a dictionary!")
        self.exact_match = exact_match

    def __str__(self):
        return "[FilterObject: Required='%s' Excluded='%s' Exact-match: %s]" % (str(self.required), str(self.excluded), str(self.exact_match))

    def dict_cleanup(self, data):
        """
        Function for filtering dictionary structures (eg. dictionary where values are also dictionaries).

        :param dict data: Dictionary containing dicts as values, which you want to filter.
        :return: Filtered dictionary.
        """
        for data_key, data_value in list(data.items()):
            # TODO: Add DEBUG logging (?)
            for filter_key, filter_value in self.required.items():
                if filter_key in data_value.keys():
                    if isinstance(filter_value, str) and self.exact_match:
                        if data_value[filter_key] != filter_value:
                            del data[data_key]
                            break
                    elif isinstance(filter_value, str) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            del data[data_key]
                            break
                        if filter_value not in data_value[filter_key]:
                            del data[data_key]
                            break
                    elif isinstance(filter_value, list) and self.exact_match:
                        if data_value[filter_key] not in filter_value:
                            del data[data_key]
                            break
                    elif isinstance(filter_value, list) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            del data[data_key]
                            break
                        found_match = False
                        for filter_value_item in filter_value:
                            if filter_value_item in data_value[filter_key]:
                                found_match = True
                        if not found_match:
                            del data[data_key]
                            break
                    else:
                        self.logger.warning(msg="Dict_Cleanup: None of the cases matched. Data: %s Filter: %s" % (data_value, self.filter))
                        # TODO: Handle other possible cases
                else:
                    self.logger.warning(msg="Dict_Cleanup: Filter key: %s not present in Data: %s" % (filter_key, data_value))
                    continue

        for data_key, data_value in list(data.items()):
            # TODO: Add DEBUG logging (?)
            for filter_key, filter_value in self.excluded.items():
                if filter_key in data_value.keys():
                    if isinstance(filter_value, str) and self.exact_match:
                        if data_value[filter_key] == filter_value:
                            del data[data_key]
                            break
                    elif isinstance(filter_value, str) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            continue
                        if filter_value in data_value[filter_key]:
                            del data[data_key]
                            break
                    elif isinstance(filter_value, list) and self.exact_match:
                        if data_value[filter_key] in filter_value:
                            del data[data_key]
                            break
                    elif isinstance(filter_value, list) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            continue
                        found_match = False
                        for filter_value_item in filter_value:
                            if filter_value_item in data_value[filter_key]:
                                found_match = True
                        if found_match:
                            del data[data_key]
                            break
                    else:
                        self.logger.warning(msg="Dict_Cleanup: None of the cases matched. Data: %s Filter: %s" % (data_value, self.filter))
                        # TODO: Handle other possible cases
                else:
                    self.logger.warning(msg="Dict_Cleanup: Filter key: %s not present in Data: %s" % (filter_key, data_value))
                    continue
        return data

    def list_cleanup(self, data):
        """
        Function for filtering list structures (eg. list of dictionaries).

        :param list data: List of dictionaries, which you want to filter.
        :return: Filtered list of dictionaries.
        """
        for data_value in list(data):
            # TODO: Add DEBUG logging (?)
            for filter_key, filter_value in self.required.items():
                if filter_key in data_value.keys():
                    if isinstance(filter_value, str) and self.exact_match:
                        if data_value[filter_key] != filter_value:
                            data.remove(data_value)
                            break
                    elif isinstance(filter_value, str) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            data.remove(data_value)
                            break
                        if filter_value not in data_value[filter_key]:
                            data.remove(data_value)
                            break
                    elif isinstance(filter_value, list) and self.exact_match:
                        if data_value[filter_key] not in filter_value:
                            data.remove(data_value)
                            break
                    elif isinstance(filter_value, list) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            data.remove(data_value)
                            break
                        found_match = False
                        for filter_value_item in filter_value:
                            if filter_value_item in data_value[filter_key]:
                                found_match = True
                        if not found_match:
                            data.remove(data_value)
                            break
                    else:
                        self.logger.warning(msg="List_Cleanup: None of the cases matched. Data: %s Filter: %s" % (data_value, self.filter))
                        # TODO: Handle other possible cases
                else:
                    self.logger.warning(msg="List_Cleanup: Filter key: %s not present in Data: %s" % (filter_key, data_value))
                    continue

        for data_value in list(data):
            # TODO: Add DEBUG logging (?)
            for filter_key, filter_value in self.excluded.items():
                if filter_key in data_value.keys():
                    if isinstance(filter_value, str) and self.exact_match:
                        if data_value[filter_key] == filter_value:
                            data.remove(data_value)
                            break
                    elif isinstance(filter_value, str) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            continue
                        if filter_value in data_value[filter_key]:
                            data.remove(data_value)
                            break
                    elif isinstance(filter_value, list) and self.exact_match:
                        if data_value[filter_key] in filter_value:
                            data.remove(data_value)
                            break
                    elif isinstance(filter_value, list) and (not self.exact_match):
                        if data_value[filter_key] is None:
                            continue
                        found_match = False
                        for filter_value_item in filter_value:
                            if filter_value_item in data_value[filter_key]:
                                found_match = True
                        if found_match:
                            data.remove(data_value)
                            break
                    else:
                        self.logger.warning(msg="List_Cleanup: None of the cases matched. Data: %s Filter: %s" % (data_value, self.filter))
                        # TODO: Handle other possible cases
                else:
                    self.logger.warning(msg="List_Cleanup: Filter key: %s not present in Data: %s" % (filter_key, data_value))
                    continue

        return data

    def universal_cleanup(self, data=None):
        """
        This function calls proper cleanup function base on data type.

        :param data: Data to be filtered, either list of dictionaries or dictionary of dictionaries.
        :return: Filtered data
        """
        data = copy.deepcopy(data)
        output = None
        if isinstance(data, list):
            output = self.list_cleanup(data=data)
        elif isinstance(data, dict):
            output = self.dict_cleanup(data=data)
        else:
            self.logger.error(msg="Universal_Cleanup: Given data is neither list nor dict.")
        return output

class OutputFilter:
    """
    This class helps to minimize dictionary structure by specifying only the desired keys.
    """
    def __init__(self, data=None, required=[], excluded=[]):
        """

        :param data: Data to be filtered.
        :param list required: List of required keys. Returned entries will contain only these specified keys. Example: `{"key1": "value1", "key2": "value2"}` \
               with ``required`` `["key1"]` will only return `{"key1": "value1"}`.

        :param list excluded: List of excluded keys. Returned entries will not contain these specified keys. Example: `{"key1": "value1", "key2": "value2"}` \
               with ``excluded`` `["key1"]` will only return `{"key2": "value2"}`.
        """
        self.logger = get_logger(name="OutputFilter")
        self.data = copy.deepcopy(data)
        self.required = None
        self.excluded = None
        if isinstance(required, list):
            self.required = required
        else:
            self.required = []
            self.logger.error(msg="Required is not a list!")
        if isinstance(excluded, list):
            self.excluded = excluded
        else:
            self.excluded = []
            self.logger.error(msg="Excluded is not a list!")

    def get(self):
        """
        After instantiating object, call this function to retrieve filtered data.

        :return: Filtered data.
        """
        if isinstance(self.data, dict):
            for id, data in list(self.data.items()):
                if len(self.required) > 0:
                    # Loop over required keys
                    for key in list(data.keys()):
                        if key not in self.required:
                            del self.data[id][key]
                elif len(self.excluded) > 0:
                    # Loop over excluded keys
                    for key in list(data.keys()):
                        if key in self.excluded:
                            del self.data[id][key]
                else:
                    continue
            return self.data

        elif isinstance(self.data, list):
            for i in range(len(self.data)):
                if isinstance(self.data[i], dict):
                    if len(self.required) > 0:
                        for key in list(self.data[i].keys()):
                            if key not in self.required:
                                del self.data[i][key]
                    elif len(self.blocked) > 0:
                        for key in list(self.data[i].keys()):
                            if key in self.excluded:
                                del self.data[i][key]
                    else:
                        continue
            return self.data

        else:
            self.logger.error(msg="Data is not a dict or list!")
            return None


from threading import Thread
import socket
host = input('host > ')
from_port = input('start scan from port > ')
to_port = input('finish scan to port > ')
counting_open = []
counting_close = []
threads = []

def scan(port):
	s = socket.socket()
	result = s.connect_ex((host,port))
	print('working on port > '+(str(port)))
	if result == 0:
		counting_open.append(port)
		#print((str(port))+' -> open')
		s.close()
	else:
		counting_close.append(port)
		#print((str(port))+' -> close')
		s.close()

for i in range(from_port, to_port+1):
	t = Thread(target=scan, args=(i,))
	threads.append(t)
	t.start()

[x.join() for x in threads]

print(counting_open)



from nuaal.connections.cli import Cisco_IOS_Cli
from nuaal.utils import int_name_convert, mac_addr_convert, interface_split, vlan_range_expander, vlan_range_shortener, Filter, OutputFilter
from nuaal.Models.BaseModels import BaseModel, DeviceBaseModel
import json
import copy


class CiscoIOSModel(DeviceBaseModel):
    """
    High-level abstraction model for Cisco IOS devices.
    """
    def __init__(self, cli_connection=None, DEBUG=False):
        """

        :param cli_connection: Instance of ``Cisco_IOS_Cli`` connection object.
        :param bool DEBUG: Enables/disables debugging output.
        """
        super(CiscoIOSModel, self).__init__(name="CiscoIOSCliModel", DEBUG=DEBUG)
        self.cli_connection = cli_connection
        self.physical_interfaces = ["FastEthernet", "GigabitEthernet", "TenGigabitEthernet"]

    def _map_interface(self, interface):
        """
        Internal function for changing the format of interface dictionary representation.

        :param dict interface: Dictionary representation of interface in a format provided by connection object's ``get_interfaces()`` function.
        :return: Dictionary representation interface in common format.
        """
        new_interface = copy.deepcopy(self.interface_model)
        new_interface["portName"] = interface["name"]
        new_interface["description"] = interface["description"]
        new_interface["interfaceType"] = "physical" if interface_split(interface["name"])[0] in self.physical_interfaces else "virtual"
        new_interface["status"] = interface["lineProtocol"]
        new_interface["adminStatus"] = interface["status"]
        new_interface["className"] = interface["hardware"]
        new_interface["macAddress"] = mac_addr_convert(interface["mac"])
        new_interface["duplex"] = interface["duplex"]
        new_interface["speed"] = interface["bandwidth"]
        new_interface["ipv4Address"] = interface["ipv4Address"]
        new_interface["ipv4Mask"] = interface["ipv4Mask"]
        new_interface["portMode"] = "routed" if interface["ipv4Address"] is not None else None

        return new_interface
    
    def _interface_update(self):
        """
        Internal function for retrieving needed data to build complete interface representation.

        :return: ``None``
        """

        if not self.cli_connection.is_alive:
            with self.cli_connection as device:
                access_vlans = device.get_vlans()
                trunks = device.get_trunks()
                interfaces = device.get_interfaces()
        else:
            access_vlans = self.cli_connection.get_vlans()
            trunks = self.cli_connection.get_trunks()
            interfaces = self.cli_connection.get_interfaces()

        #
        for interface in interfaces:
            name = interface["name"]
            self.interfaces[name] = self._map_interface(interface)

        #
        for vlan in access_vlans:
            vlan_id = vlan["id"]
            self.vlans[vlan_id] = copy.deepcopy(self.vlan_model)
            self.vlans[vlan_id]["name"] = vlan["name"]
            self.vlans[vlan_id]["vlanId"] = vlan["id"]
            self.vlans[vlan_id]["status"] = vlan["status"]

            for port in vlan["access_ports"]:

                port_long = int_name_convert(port)
                self.vlans[vlan_id]["untaggedPorts"].append(port_long)
                self.interfaces[port_long]["portMode"] = "access"
                self.interfaces[port_long]["untaggedVlanId"] = vlan_id

        for trunk in trunks:
            port = int_name_convert(trunk["interface"])
            self.interfaces[port]["portMode"] = "trunk"
            self.interfaces[port]["untaggedVlanId"] = trunk["nativeVlan"]
            self.interfaces[port]["taggedVlanIds"] = trunk["allowed"]
            if port not in self.vlans[trunk["nativeVlan"]]["untaggedPorts"]:
                self.vlans[trunk["nativeVlan"]]["untaggedPorts"].append(port)
            for vlan_id in vlan_range_expander(trunk["active"]):
                if vlan_id not in self.vlans[vlan_id]["taggedPorts"]:
                    self.vlans[vlan_id]["taggedPorts"].append(port)

    def get_interfaces(self):
        """
        Retrieves list of all interfaces in common format.

        :return: List of dictionaries.
        """
        if len(self.interfaces) == 0:
            self._interface_update()
        return self.interfaces

    def get_l3_interfaces(self):
        """
        Retrieves list of L3 (routed) interfaces in common format.

        :return: List of dictionaries.
        """
        if len(self.interfaces) == 0:
            self._interface_update()
        filter = Filter(required={"portMode": "routed"}, exact_match=True)
        l3_interfaces = filter.universal_cleanup(data=self.interfaces)
        return l3_interfaces

    def get_l2_interfaces(self):
        """
        Retrieves list of L2 (switched) interfaces in common format.


        :return: List of dictionaries.
        """
        if len(self.interfaces) == 0:
            self._interface_update()
        filter = Filter(required={"portMode": ["access", "trunk"]})
        l2_interfaces = filter.universal_cleanup(data=self.interfaces)
        return l2_interfaces

    def get_vlans(self):
        """
        Retrieves list of all configured VLANs on device in common format.

        :return: List of dictionaries.
        """
        if len(self.vlans) == 0:
            self._interface_update()
        return self.vlans

    def get_inventory(self):
        """
        Retrieves list of all installed HW parts on device in common format.

        :return: List of dictionaries.
        """
        device_inventory = []
        if not self.cli_connection.is_alive:
            with self.cli_connection as device:
                device_inventory = device.get_inventory()
        else:
            device_inventory = self.cli_connection.get_inventory()
        for raw_module in device_inventory:
            module = copy.deepcopy(self.inventory_model)
            module["name"] = raw_module["name"]
            module["description"] = raw_module["desc"]
            module["partNumber"] = raw_module["pid"]
            module["serialNumber"] = raw_module["sn"]
            module["version"] = raw_module["vid"]
            self.inventory.append(module)
        return self.inventory

